import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';
import { timer } from 'rxjs/observable/timer';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { GetDateTimePipe } from '../pipes/getdatetime.pipe';

import { ScheduleService } from './schedule.service';

import { Slot } from '../models/schedule';


export class ScheduleDataSource implements DataSource<Slot> {
    private scheduleSubject = new BehaviorSubject<Slot[]>([]);
    private loadingSubject = new BehaviorSubject<boolean>(false);
    private countdownSubject = new BehaviorSubject<number>(undefined);

    public loading$ = this.loadingSubject.asObservable();
    public countdown$ = this.countdownSubject.asObservable();

    constructor(private scheduleService: ScheduleService) { }

    loadSchedule(disabled: boolean) {
        this.loadingSubject.next(true);

        this.scheduleService.getSchedule(disabled)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe(schedule => {
                schedule = this.sorted(schedule);

                this.scheduleSubject.next(schedule);

                timer(0, 1000).subscribe(() =>
                    this.countdownSubject.next(this.countdown(schedule))
                );
            });
    }

    private sorted(schedule) {
        schedule.sort(function(a, b) {
            a = new GetDateTimePipe().transform(a);
            b = new GetDateTimePipe().transform(b);
            return a.getTime() - b.getTime();
        });

        return schedule;
    }

    private countdown(schedule) {
        if (schedule.length == 0)
            return undefined;

        let next = new GetDateTimePipe().transform(schedule[0]);
        return next.getTime() - Date.now();
    }

    connect(collectionViewer: CollectionViewer): Observable<Slot[]> {
        console.log('Connecting data source');
        return this.scheduleSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.scheduleSubject.complete();
        this.loadingSubject.complete();
        this.countdownSubject.complete();
    }
}
